% 
% Eurographics 2012 - Tutorial on "Diffusion Geometry in Shape Analysis"
%
%  Umberto Castellani, Alex Bronstein, Michael Bronstein
%

clc
close all
clear

% Read model as ply format
%
data = ply_read('models\Male_null.ply');
%
shape.X = data.vertex.x;
shape.Y = data.vertex.y;
shape.Z = data.vertex.z;
TRIV = zeros(length(data.face.vertex_indices),3);
for k = 1:length(data.face.vertex_indices)
   TRIV(k,:) = data.face.vertex_indices{k} + 1; 
end
shape.TRIV = TRIV;

% Show 3D model
%
figure(1)
trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight


% Eigendecomposition
%
% Compute matrices (see http://www.geomtop.org/software/meshlp.html)
%
[W, A] = mshlp_matrix(shape);
Am = sparse([1:length(A)], [1:length(A)], A);
warning off;
% Compute eigendecomposition
neig=200;
[evecs evals] = eigs(W, Am, neig, -1e-5);
warning on;
evals = abs(diag(evals));

abssc=0;

if (abssc)
    % Linear scales
    %
    % small scales
    %ts = 1:4:400;
    %
    % larger scales
    ts = 1:200:20000;
else
    % Nonlinear-scales (euristic from
    % http://www.geomtop.org/paper_ppt/multiscale_htker.pdf)
    %
    tmin = abs(4*log(10) / evals(end));
    tmax = abs(4*log(10) / evals(2));
    nstep = 100;
    stepsize = (log(tmax) - log(tmin)) / nstep;
    logts = log(tmin):stepsize:log(tmax);
    ts = exp(logts);
end

vis_auto=1;
vis_hks=1;

% compute autodiffusion at different scales (i.e., HKS)
hks = abs( evecs(:, 2:end) ).^2 * exp( - abs(evals(2:end)) * ts);


% Visualize the autodiffusion for all points at different scales
if(vis_auto)
    for t=1:10:length(ts)
        figure(2)
        trisurf(shape.TRIV,shape.X,shape.Y,shape.Z,hks(:,t)), shading interp, axis image, axis tight, lighting phong, camlight
        colorbar
    end
end
disp('Now we focus on each single point');
input('...');

%Visualize the HKS for each point
if (vis_hks)
    for i=1:100:length(hks)
        figure(3)
        subplot(1,2,1)
        hold off
        plot3(shape.X(i), shape.Y(i), shape.Z(i), 'r.', 'Markersize', 30);
        hold on
        trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight
        %
        subplot(1,2,2)
        plot(log(hks(i,:)), 'x');
        title('HKS');
        disp(['We are analysing point ' num2str(i)]);
        input('...');
    end
end




% Now we compare the signature of some points:
%
disp('Now we compare the signatures of some points');
i=301; % Head
figure(4)
subplot(1,2,1)
trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight
hold on
plot3(shape.X(i), shape.Y(i), shape.Z(i), 'r.', 'Markersize', 30);
hold on
title('Head');
%
subplot(1,2,2)
hold on
plot(log(hks(i,:)), 'rx');
title('HKS');
input('...');

i=145; % Right finger
subplot(1,2,1)
plot3(shape.X(i), shape.Y(i), shape.Z(i), 'b.', 'Markersize', 30);
title('Head & Right finger');
subplot(1,2,2)
hold on
plot(log(hks(i,:)), 'bx');
title('HKS');
input('...');

i=142; % Left finger
subplot(1,2,1)
plot3(shape.X(i), shape.Y(i), shape.Z(i), 'm.', 'Markersize', 30);
title('Head & Right-Left fingers');
subplot(1,2,2)
hold on
plot(log(hks(i,:)), 'mx');
title('HKS');
input('...');
        
       
% check the descriptor similarity
p1=301; %head
p2=145; %right finger
p3=142; % right finger
d_12=norm(hks(p1,:) - hks(p2,:));
d_13=norm(hks(p1,:) - hks(p3,:));
d_23=norm(hks(p2,:) - hks(p3,:));

disp(['Head-finger 1: ' num2str(d_12)]);
disp(['Head-finger 2: ' num2str(d_13)]);
disp(['Finger 1- Finger 2: ' num2str(d_23)]);

figure(5)
bar([d_12 d_13 d_23]);
xlabel('1:err(Head-finger1)      2:err(Head finger2)      3:err(Finger1-Finger2)');
ylabel('error');
  
    